/**
 * @file
 *
 * Memory management; will support NUMA if available and fall back
 * to malloc() otherwise.
 *
 * @author Jens Teubner <jens.teubner@inf.ethz.ch>
 *
 * (c) 2010 ETH Zurich, Systems Group
 *
 * $Id: mem.c 1100 2011-08-17 14:32:58Z jteubner $
 */
#include "config.h"

#include <assert.h>
#include <stdlib.h>
#include <stdio.h>

#include "mem/mem.h"

#ifdef HAVE_LIBNUMA
#include <numa.h>
#endif

#ifdef HAVE_LIBNUMA

void
nb_free (void *ptr, size_t size)
{
    numa_free (ptr, size);
}

void *
nb_alloc (size_t size, struct bitmask *nodemask)
{
    void *ret;

    ret = numa_alloc_interleaved_subset (size, nodemask);

    if (! ret)
    {
        fprintf (stderr, "Error allocating NUMA memory\n");
        exit (EXIT_FAILURE);
    }

    return ret;
}

#else  /* HAVE_LIBNUMA? */

void
nb_free (void *ptr, size_t size __attribute__((unused)))
{
    free (ptr);
}

void *
nb_alloc (size_t size, void *nodemask __attribute__((unused)))
{
    void *ret = malloc (size);

    if (! ret)
    {
        fprintf (stderr, "Error allocating memory\n");
        exit (EXIT_FAILURE);
    }

    return ret;
}

#endif  /* HAVE_LIBNUMA? */
