/**
 * @file
 *
 * Worker threads perform memory scanning and benchmarking; possibly
 * many workers run in parallel.
 *
 * @author Jens Teubner <jens.teubner@inf.ethz.ch>
 *
 * (c) 2010 ETH Zurich, Systems Group
 *
 * $Id: worker.c 1100 2011-08-17 14:32:58Z jteubner $
 */

#include "config.h"

#include <stdio.h>

#ifdef HAVE_LIBNUMA
#include <numa.h>
#endif

#include "worker/worker.h"
#include "workset/workset.h"
#include "setup/setup.h"

void *
scan_worker (void *arg)
{
    thread_t   *ctx = (thread_t *) arg;

#ifdef HAVE_LIBNUMA
    /* TODO: migrate to given NUMA node */
    if (numa_run_on_node (ctx->numa_node))
    {
        fprintf (stderr, "Error migrating thread to NUMA node %i.\n",
            ctx->numa_node);
        exit (EXIT_FAILURE);
    }
#endif

    if (ctx->mode == mode_readwrite
            && *(ctx->region->stride) < 2 * sizeof (elem_t))
    {
        fprintf (stderr,
                "Stride width %li is too small for read/write access.\n",
                *(ctx->region->stride));
        exit (EXIT_FAILURE);
    }

    for (unsigned int i = 0; i < ctx->num_runs; i++)
        ctx->nsec_per_step[i] = scan (ctx->region->workset, ctx->mode);

    return NULL;
}
