/**
 * @file
 *
 * Worker threads perform memory scanning and benchmarking; possibly
 * many workers run in parallel.
 *
 * @author Jens Teubner <jens.teubner@inf.ethz.ch>
 *
 * (c) 2010 ETH Zurich, Systems Group
 *
 * $Id: worker.h 1100 2011-08-17 14:32:58Z jteubner $
 */

#include "config.h"

#ifndef WORKER_H
#define WORKER_H

#include <stdlib.h>
#include <pthread.h>

#include "workset/workset.h"

/**
 * We pass a struct of this type to the worker thread, so it knows
 * what to do.
 */
struct worker_ctx_t {

    /** pthreads handler for this thread */
    pthread_t     thread;

    /** NUMA node to run this thread on */
    int           numa_node;

    /** working set to run the scan on; must have been prepared before */
    elem_t       *workset;

    /** number of scan() executions */
    unsigned int  num_runs;

    /**
     * the worker will write its results in here; this must have been
     * allocated as an array with @a num_runs items
     */
    double       *nsec_per_step;
};

typedef struct worker_ctx_t worker_ctx_t;

void * scan_worker (void *arg);

#endif  /* WORKER_H? */
