/**
 * @file
 *
 * Scan a given workset.
 *
 * @author Jens Teubner <jens.teubner@inf.ethz.ch>
 *
 * (c) 2010 ETH Zurich, Systems Group
 *
 * $Id: scan.c 1100 2011-08-17 14:32:58Z jteubner $
 */

#include "config.h"

#include <stdio.h>

#include "workset/workset.h"
#include "time/time.h"

#if 0
#define ONE         e = e->next;
#define FIVE        ONE ONE ONE ONE ONE
#define TEN         FIVE FIVE
#define FIFTY       TEN TEN TEN TEN TEN
#define HUNDRED     FIFTY FIFTY
#define FIVEHUNDRED HUNDRED HUNDRED HUNDRED HUNDRED HUNDRED
#define THOUSAND    FIVEHUNDRED FIVEHUNDRED
#endif

#define ONE(o)         o
#define FIVE(o)        ONE(o); ONE(o); ONE(o); ONE(o); ONE(o)
#define TEN(o)         FIVE(o); FIVE(o)
#define FIFTY(o)       TEN(o); TEN(o); TEN(o); TEN(o); TEN(o)
#define HUNDRED(o)     FIFTY(o); FIFTY(o)
#define FIVEHUNDRED(o) HUNDRED(o); HUNDRED(o); HUNDRED(o); HUNDRED(o); HUNDRED(o)
#define THOUSAND(o)    FIVEHUNDRED(o); FIVEHUNDRED(o)

static elem_t *dummy;

/**
 * Scan the given working set and return the average time per
 * iteration (in nano-seconds).
 */
double
scan (elem_t *workset, accessmode_t mode)
{
    struct timespec  t_start;
    struct timespec  t_end;
    elem_t          *e = workset;
    unsigned int     i;

    double           nsec;

    nb_gettime (&t_start);

    if (mode == mode_readonly)
        for (i = 0; i < SCAN_ITERATIONS; i++)
        {
            /* WARNING! Don't remove the braces! */
            THOUSAND( e = e->next );
        }
    else
        for (i = 0; i < SCAN_ITERATIONS; i++)
        {
            /* WARNING! Don't remove the braces! */
            THOUSAND( ((e + 1)->next)++ ; e = e->next );
        }

    nb_gettime (&t_end);

    dummy = e;

    nsec = (t_end.tv_sec * 1000000000L + t_end.tv_nsec)
        - (t_start.tv_sec * 1000000000L + t_start.tv_nsec);

    return nsec / (1000 * SCAN_ITERATIONS);
}
